#line 2 "ada-lex.c"
/* A lexical scanner generated by flex */

/* Scanner skeleton version:
 * $Header: /home/daffy/u0/vern/flex/RCS/flex.skl,v 2.91 96/09/10 16:58:48 vern Exp $
 */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5

#include <stdio.h>
#include <unistd.h>


/* cfront 1.2 defines "c_plusplus" instead of "__cplusplus" */
#ifdef c_plusplus
#ifndef __cplusplus
#define __cplusplus
#endif
#endif


#ifdef __cplusplus

#include <stdlib.h>

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#if __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */

#ifdef __TURBOC__
 #pragma warn -rch
 #pragma warn -use
#include <io.h>
#include <stdlib.h>
#define YY_USE_CONST
#define YY_USE_PROTOS
#endif

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct yy_buffer_state *YY_BUFFER_STATE;

extern int yyleng;
extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 * 	if ( condition_holds )
 *		yyless( 5 );
 *	else
 *		do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the yyless() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yytext_ptr )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */
typedef unsigned int yy_size_t;


struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can xrealloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};

static YY_BUFFER_STATE yy_current_buffer = 0;

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer


/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;

static int yy_n_chars;		/* number of characters read into yy_ch_buf */


int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart YY_PROTO(( FILE *input_file ));

void yy_switch_to_buffer YY_PROTO(( YY_BUFFER_STATE new_buffer ));
void yy_load_buffer_state YY_PROTO(( void ));
YY_BUFFER_STATE yy_create_buffer YY_PROTO(( FILE *file, int size ));
void yy_delete_buffer YY_PROTO(( YY_BUFFER_STATE b ));
void yy_init_buffer YY_PROTO(( YY_BUFFER_STATE b, FILE *file ));
void yy_flush_buffer YY_PROTO(( YY_BUFFER_STATE b ));
#define YY_FLUSH_BUFFER yy_flush_buffer( yy_current_buffer )

YY_BUFFER_STATE yy_scan_buffer YY_PROTO(( char *base, yy_size_t size ));
YY_BUFFER_STATE yy_scan_string YY_PROTO(( yyconst char *yy_str ));
YY_BUFFER_STATE yy_scan_bytes YY_PROTO(( yyconst char *bytes, int len ));

static void *yy_flex_alloc YY_PROTO(( yy_size_t ));
static void *yyxrealloc YY_PROTO(( void *, yy_size_t ));
static void yy_flex_free YY_PROTO(( void * ));

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (yy_current_buffer->yy_at_bol)

typedef unsigned char YY_CHAR;
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;
typedef int yy_state_type;
extern char *yytext;
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state YY_PROTO(( void ));
static yy_state_type yy_try_NUL_trans YY_PROTO(( yy_state_type current_state ));
static int yy_get_next_buffer YY_PROTO(( void ));
static void yy_fatal_error YY_PROTO(( yyconst char msg[] ));

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;

#define YY_NUM_RULES 52
#define YY_END_OF_BUFFER 53
static yyconst short int yy_accept[336] =
    {   0,
        0,    0,    0,    0,    0,    0,   53,   51,    1,    1,
       14,   50,   40,   51,   42,   43,   40,   41,   40,   40,
       40,    4,    4,   40,   40,   40,   40,   49,   46,   46,
       46,   46,   46,   46,   46,   46,   46,   46,    1,   51,
       15,   50,   40,   51,   42,   43,   40,   41,   40,   40,
       40,    4,    4,   40,   40,   40,   40,   49,   40,   46,
       46,   46,   46,   46,   46,   46,   46,   46,   46,   39,
       50,    0,    0,    0,    0,    0,    0,    0,   34,    2,
        0,   33,    0,   45,   45,   36,    0,    0,    4,    0,
        0,   35,   37,    0,   32,   38,    0,    0,   46,    0,

       46,    0,   46,   46,   46,   18,   22,   46,   46,   46,
       46,   27,   46,   46,   46,    0,   15,    0,   50,    0,
        0,   15,    0,    0,   34,    2,    0,   33,    0,   45,
       45,   36,    0,    0,    4,    0,    0,   35,   37,    0,
       32,   38,   15,    0,   46,    0,   46,    0,   46,   46,
       46,   18,   22,   46,   46,   46,   46,   27,   46,   46,
       46,    0,    0,   12,   12,    0,   31,    2,    0,   45,
       45,    0,    9,    0,    3,    7,    0,   46,    0,    0,
        0,   46,   48,   19,   20,   46,   23,   24,   25,   46,
       28,   46,   30,    0,   15,   31,    2,    2,    2,    0,

       45,   45,    0,    9,    0,    3,    7,    0,   46,    0,
        0,    0,   15,    0,   46,   48,   19,   20,   46,   23,
       24,   25,   46,   28,   46,   30,    0,    0,    0,    0,
       45,   44,    6,    0,    0,    9,    0,    3,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
       46,   21,   26,   29,    0,    0,    0,    0,    2,    0,
       45,   44,    6,    0,    0,    9,    0,    3,    0,    0,
        0,   46,   21,   26,   29,   47,    0,    0,    0,    0,
       46,    0,    0,    0,    0,    0,    0,    0,   47,    0,
        0,    2,    2,    0,    0,    0,   17,    0,    0,   46,

        0,    0,    5,   11,    0,    8,   17,    0,    2,    2,
        0,    5,   11,    0,    8,   16,    0,    5,    0,    8,
       16,    2,    2,    5,    0,    8,   13,    0,    2,    0,
       10,   10,   10,   10,    0
    } ;

static yyconst int yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    4,    5,    6,    7,    8,    5,    9,   10,   11,
       12,   13,   14,   15,   16,   17,   18,   19,   20,   20,
       20,   20,   20,   20,   20,   20,   20,   21,   22,   23,
       24,   25,    5,   26,   30,   31,   32,   33,   34,   35,
       36,   37,   38,   36,   36,   39,   40,   41,   42,   36,
       36,   43,   44,   45,   46,   36,   47,   48,   36,   36,
       27,    5,   28,    5,   29,    5,   30,   31,   32,   33,

       34,   35,   36,   37,   38,   36,   36,   39,   40,   41,
       42,   36,   36,   43,   44,   45,   46,   36,   47,   48,
       36,   36,   26,   22,   26,    5,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst int yy_meta[49] =
    {   0,
        1,    2,    3,    4,    5,    6,    7,    8,    5,    9,
        5,    5,    5,    5,    5,    5,   10,    5,   11,   11,
        9,    5,   12,   13,   14,    5,    5,    5,   15,   16,
       16,   16,   16,   16,   16,   17,   17,   17,   17,   17,
       17,   17,   17,   17,   17,   17,   17,   17
    } ;

static yyconst short int yy_base[358] =
    {   0,
        0,    0,   48,    0,   91,   92, 1294, 1747, 1747, 1747,
     1747,    0, 1747,  102, 1747, 1747, 1280, 1747, 1276,  149,
     1245,  148,  154, 1244, 1243, 1241, 1241, 1747,  182,  202,
      123,  128,  139,  156,  170,  176,  177,  184,  151,  189,
     1747,  240,  194,  288,  203,  214,  331,  332,  333,  359,
      402,  424, 1216,  408,  469,  409,  413,  406,  411,  516,
      191, 1224,  193, 1220,  193, 1194, 1202, 1198, 1188,  564,
        0,  611, 1218, 1216, 1197, 1196,  150,  650, 1747,    0,
      697, 1747,    0,    0, 1166, 1747,    0,   80,  435,  217,
        0, 1747, 1747, 1176, 1747, 1747,  407,  446, 1747,  614,

      309, 1174,  310,  312,  311,  315,  316,  429,  588,  382,
      591,  416,  593,  597,  630,  412, 1747,  419,    0, 1178,
      726,    0, 1160,  745,  439,  704, 1148,  696,  770,  800,
     1125,  697,  843,  698, 1113,  720,  860,  705,  706,  892,
      710,  715, 1099,  939,  716,  965,    0,  844, 1060, 1070,
     1058,    0,    0, 1068, 1050, 1040, 1029,    0, 1025, 1017,
     1006, 1038,  851,  864, 1747,  995, 1011,    0,  994,    0,
      925,  454,  643,  156,  607,    0,  938,  944, 1014, 1029,
        0, 1029, 1747,  633,  634,  635,  637,  678,  704,  707,
      710,  713,  820, 1054, 1076,    0, 1034,    0, 1061, 1108,

        0,  920, 1150, 1060, 1072, 1167, 1178,    0, 1212,  952,
        0,    0,    0, 1211, 1241,  854,    0,    0,  923,    0,
        0,    0,  916,    0,  910,    0, 1020,  929, 1271,  919,
     1747,    0,  855,    0,  852, 1064,  175,  933,  882,  702,
      218,  858, 1049, 1003,  845,  844,  841,  849,  816,  831,
     1217, 1033, 1045, 1059, 1172, 1196,  819, 1288, 1318,    0,
     1162,    0, 1197, 1335,    0, 1306, 1330, 1352,  796,  792,
     1378,    0,    0,    0,    0, 1747,  791,  402,  789,  174,
     1425,  751,  722,  720,  707,  673,  664,  654, 1165,  642,
      661,  619, 1428, 1424, 1457, 1290, 1747,  595,    0, 1491,

      546,  620, 1143,  434,  458, 1171,    0,  431,  392,  342,
     1430, 1446, 1198,    0, 1490, 1747,  219, 1310,  224, 1312,
        0,    0,  130, 1494, 1510, 1509, 1156,  730,    0, 1479,
     1344, 1514, 1412, 1525, 1747, 1547, 1563, 1568, 1584, 1598,
     1615, 1618, 1625,  810,  811, 1632, 1648, 1665, 1677, 1683,
     1694, 1696, 1712,  812,  841, 1719, 1730
    } ;

static yyconst short int yy_def[358] =
    {   0,
      335,    1,  335,    3,    1,    1,  335,  335,  335,  335,
      335,  336,  335,  335,  335,  335,  335,  335,  335,  337,
      335,  335,  335,  335,  338,  335,  335,  335,  339,  339,
       30,   30,   30,   30,   30,   30,   30,   30,  340,  340,
      335,  335,  340,  335,  340,  340,  340,  340,  340,  335,
      340,  340,   52,  340,  335,  340,  340,  340,  340,  335,
       60,   60,   60,   60,   60,   60,   60,   60,   60,  335,
      336,   70,   70,   70,   72,  335,   72,   72,  335,  341,
      337,  335,  342,  343,  343,  335,  344,  335,  335,  335,
      345,  335,  335,  346,  335,  335,  335,  335,  335,  347,

       30,  335,   30,   30,   30,   30,   30,   30,   30,   30,
       30,   30,   30,   30,   30,  340,  335,  340,   42,   44,
       44,   72,  121,  121,  340,  348,   50,  340,   55,  129,
      130,  340,  340,  340,   52,  340,  133,  340,  340,  335,
      340,  340,  349,  340,  340,  335,   60,  340,   60,   60,
       60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
       60,   78,  335,  335,  335,   72,   78,  341,  350,  343,
      343,  351,  335,  335,  335,  345,  346,  335,  347,  335,
      352,  353,  335,   30,   30,   30,   30,   30,   30,   30,
       30,   30,   30,  340,   72,  124,  348,  341,  348,  335,

      130,  130,  133,  340,  340,  340,  133,  140,  340,  335,
      354,  146,  180,  129,   60,  340,   60,   60,   60,   60,
       60,   60,   60,   60,   60,   60,  335,  335,   72,  350,
      335,  343,  335,  355,  351,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  356,
      353,   30,   30,   30,  340,  340,   72,   72,  341,  200,
      340,  130,  340,  133,  203,  340,  340,  340,  335,  335,
      335,  215,   60,   60,   60,  335,   72,  335,  357,  335,
      335,  335,  335,  335,  335,  335,  335,  356,  340,   72,
       72,  341,  341,  340,  133,  340,  335,  335,  271,  340,

       72,  335,  335,  335,  357,  335,   72,   72,  341,  341,
      340,  340,  340,  295,  340,  335,   72,  335,  335,  335,
       72,  341,  341,  340,  340,  340,  335,  335,  341,  340,
      335,  340,  335,  340,    0,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335
    } ;

static yyconst short int yy_nxt[1796] =
    {   0,
        8,    9,   10,    9,    8,   11,    8,   12,   13,   14,
       15,   16,   17,   13,   18,   19,   20,   21,   22,   23,
       24,   13,   25,   26,   27,   28,   13,   13,   29,   30,
       29,   29,   29,   31,   29,   29,   29,   32,   29,   33,
       34,   35,   36,   29,   37,   29,   29,   38,    8,    9,
       10,   39,   40,   41,   40,   42,   43,   44,   45,   46,
       47,   43,   48,   49,   50,   51,   52,   53,   54,   43,
       55,   56,   57,   58,   59,   43,   60,   61,   60,   60,
       60,   62,   60,   60,   60,   63,   60,   64,   65,   66,
       67,   60,   68,   60,   60,   69,    8,    8,  173,  173,

       70,   70,   72,   73,   73,   74,   75,   75,   75,   75,
       75,   76,   75,   75,   75,   75,   75,   75,   75,   75,
       75,   75,   75,   75,   75,   75,   75,   75,   77,   75,
       75,   78,   78,   78,   78,   78,   78,   78,   78,   78,
       78,   78,   78,   78,   78,   78,   78,   78,   78,   78,
       81,   81,   81,  101,   87,  166,  117,  329,  101,  164,
       87,  105,  106,  101,   88,   82,   89,   89,  107,  101,
       88,   83,   89,   89,  175,  175,   89,  118,   85,  101,
      108,   90,   89,   97,   97,   98,  101,   90,  280,  109,
      280,   99,  306,  306,  117,   91,  101,  110,  100,  117,

      101,  111,  102,   97,   97,   98,  101,  101,  117,  113,
      101,   99,  112,  114,  101,  118,  101,  101,  100,  117,
      118,  149,  102,  281,  101,  115,  155,  152,  327,  118,
      174,  150,  103,  153,  156,  175,  175,  328,  157,  328,
      118,  239,  104,  116,  116,  117,  116,  119,  116,  116,
      116,  116,  116,  116,  116,  116,  116,  116,  119,  119,
      116,  116,  116,  116,  116,  116,  118,  116,  119,  119,
      119,  119,  119,  119,  119,  119,  119,  119,  119,  119,
      119,  119,  119,  119,  119,  119,  119,  119,   72,   73,
       73,  120,  121,  122,  121,  121,  121,  116,  121,  121,

      121,  121,  121,  121,  121,  121,  121,  121,  121,  121,
      121,  121,  121,  121,  123,  121,  121,  124,  124,  124,
      124,  124,  124,  124,  124,  124,  124,  124,  124,  124,
      124,  124,  124,  124,  124,  124,  117,  117,  117,  101,
      101,  101,  101,  125,  185,  101,  101,  323,  126,  101,
      101,  101,  101,  184,  186,  101,  101,  118,  118,  118,
       81,   81,  127,  116,  117,  116,  116,  116,  116,  116,
      116,  116,  116,  116,  116,  128,  116,  116,  116,  116,
      116,  129,  116,  116,  116,  118,  116,  130,  131,  130,
      130,  130,  130,  130,  130,  130,  130,  130,  130,  130,

      130,  130,  130,  130,  130,  130,  130,  117,   97,   97,
       97,  117,  101,  117,  117,  302,  143,  117,  117,  322,
      303,  303,  101,  100,  143,  132,  189,  102,  118,  117,
      133,  138,  118,  141,  118,  118,  142,  118,  118,  118,
      134,   87,  135,  135,  117,  118,  101,   97,   97,   98,
      118,   88,  135,   89,   89,   99,  101,  136,  321,  101,
      233,  187,  100,   89,  304,  118,  102,  319,   90,  101,
      234,  137,  116,  116,  117,  116,  116,  116,  116,  116,
      116,  116,  116,  116,  116,  116,  116,  116,  116,  116,
      116,  116,  139,  116,  116,  118,  116,  140,  140,  140,

      140,  140,  140,  140,  140,  140,  140,  140,  140,  140,
      140,  140,  140,  140,  140,  140,  140,   97,   97,  144,
      116,  117,  116,  116,  116,  145,  116,  116,  116,  116,
      116,  116,  146,  116,  147,  147,  148,  116,  116,  116,
      116,  116,  118,  116,  147,  147,  147,  147,  147,  147,
      147,  147,  147,  147,  147,  147,  147,  147,  147,  147,
      147,  147,  147,  147,   72,   73,   73,   73,   72,   72,
       72,   72,   72,  317,   72,   72,   72,   72,   72,   72,
       72,   72,   72,   72,   72,   72,   72,   72,   72,   72,
       72,   72,   72,  162,  162,  162,  162,  162,  162,  162,

      162,  162,  162,  162,  162,  162,  162,  162,  162,  162,
      162,  162,   72,   72,   72,  179,  179,  179,  101,  180,
      163,  101,  316,  101,  309,  238,  238,  101,  101,  190,
      192,  101,  191,  101,  188,  238,  181,  101,  303,  303,
       72,   72,   72,   72,   72,   72,   72,   72,   72,   72,
       72,   72,   72,   72,   72,   72,   72,   72,   72,  164,
      101,  236,  236,  101,  101,  101,  308,  101,  252,  307,
      101,  236,  193,  101,  101,  101,  237,  101,  281,  167,
      167,  167,  167,  167,  167,  167,  167,  167,  167,  167,
      167,  167,  167,  167,  167,  167,  167,  167,   81,   81,

       81,  117,  117,  117,  168,  168,  239,  281,  101,  198,
      117,  117,  239,  335,  239,  117,  204,  204,  101,   83,
      117,  117,  118,  118,  118,  117,   85,   72,   72,  121,
      199,  118,  118,  205,  101,  194,  118,  101,  206,  206,
      101,  118,  118,  101,  101,  253,  118,  101,  331,  331,
      101,  239,  239,  254,  239,  121,  121,  121,  121,  121,
      121,  121,  121,  121,  121,  121,  121,  121,  121,  121,
      121,  121,  121,  121,  196,  196,  196,  196,  196,  196,
      196,  196,  196,  196,  196,  196,  196,  196,  196,  196,
      196,  196,  196,  116,  239,  304,  301,  298,  200,  200,

      200,  200,  200,  200,  200,  200,  200,  200,  200,  200,
      200,  200,  200,  200,  200,  200,  200,  200,  201,  201,
      172,  176,  270,  297,  290,  172,  176,  270,  201,  201,
      201,  201,  201,  201,  201,  201,  201,  201,  201,  201,
      201,  201,  201,  201,  201,  201,  201,  201,  117,  117,
      101,  279,  227,  227,  227,  281,  279,  287,  233,  117,
      101,  203,  203,  281,  216,  227,  227,  227,  234,  118,
      118,  228,  203,  203,  203,  203,  203,  203,  207,  207,
      118,  239,  286,  239,  228,  285,  284,  281,  278,  207,
      207,  207,  207,  207,  207,  116,  116,  117,  116,  116,

      116,  116,  116,  116,  116,  116,  116,  116,  116,  116,
      208,  208,  116,  116,  116,  116,  209,  116,  118,  116,
      208,  208,  208,  208,  208,  208,  208,  208,  208,  208,
      208,  208,  208,  208,  208,  208,  208,  208,  208,  208,
       97,   97,  144,  231,  117,   97,   97,   98,  145,  276,
      275,  238,  238,   99,  274,  146,  273,  269,  262,  148,
      100,  238,  178,  232,  102,  118,  179,  179,  212,  116,
      213,  116,  116,  116,  116,  116,  116,  116,  116,  116,
      116,  116,  116,  116,  116,  116,  116,  214,  116,  116,
      116,  118,  116,  215,  215,  215,  215,  215,  215,  215,

      215,  215,  215,  215,  215,  215,  215,  215,  215,  215,
      215,  215,  215,  229,  229,  179,  179,  179,  231,  180,
      163,  227,  227,  227,  229,  229,  229,  229,  229,  229,
       97,   97,   98,  282,  168,  168,  181,  239,   99,  198,
      228,  240,  239,  283,  239,  100,  241,  163,  226,  102,
      225,  242,  239,  243,  281,  227,  227,  255,  244,  117,
      199,  168,  168,  101,  224,  117,  259,  223,  245,  246,
      247,  248,  239,  101,  256,  101,  249,  117,  266,  266,
      118,  257,  236,  236,  222,  101,  118,  199,  266,  101,
      206,  206,  236,  267,  258,  258,  221,  237,  118,  101,

      220,  219,  218,  217,  210,  258,  258,  258,  258,  258,
      258,  116,  116,  117,  116,  116,  116,  116,  116,  116,
      116,  116,  116,  116,  116,  116,  260,  260,  116,  116,
      116,  116,  261,  116,  118,  116,  260,  260,  260,  260,
      260,  260,  260,  260,  260,  260,  260,  260,  260,  260,
      260,  260,  260,  260,  260,  260,  263,  227,  227,  227,
      116,  318,  318,  202,  116,  195,  264,  117,  265,  265,
      117,  318,  117,  227,  227,  255,  228,  117,  265,  265,
      265,  265,  265,  265,  265,  268,  268,  194,  118,  320,
      320,  118,  256,  118,  183,  268,  207,  207,  118,  320,

      178,  117,  117,  117,  171,  165,  164,  207,  207,  207,
      207,  207,  207,   97,   97,  144,  289,  117,   97,   97,
       98,  145,  118,  118,  118,  164,   99,  163,  146,  161,
      294,  325,  148,  100,  160,  159,  158,  102,  118,  271,
      271,  271,  271,  271,  271,  271,  271,  271,  271,  271,
      271,  271,  271,  271,  271,  271,  271,  271,  271,  272,
      272,  154,  151,  116,   96,   95,   93,   92,   86,  272,
      272,  272,  272,  272,  272,  272,  272,  272,  272,  272,
      272,  272,  272,  272,  272,  272,  272,  272,  272,  277,
      277,   80,   79,  335,  335,  117,  335,  335,  335,  335,

      277,  277,  277,  277,  277,  277,  291,  291,  315,  315,
      335,  117,  335,  335,  335,  335,  118,  291,  291,  291,
      291,  291,  291,  292,  266,  266,  335,  335,  318,  318,
      320,  320,  118,  335,  266,  117,  293,  293,  318,  267,
      320,  335,  335,  296,  335,  296,  335,  293,  293,  293,
      293,  293,  293,  295,  295,  335,  118,  117,  335,  335,
      335,  335,  333,  333,  295,  295,  295,  295,  295,  295,
      268,  268,  333,  335,  335,  335,  335,  335,  118,  335,
      268,  116,  116,  117,  116,  116,  116,  116,  116,  116,
      116,  116,  116,  116,  116,  116,  299,  299,  116,  116,

      116,  116,  300,  116,  118,  116,  299,  299,  299,  299,
      299,  299,  299,  299,  299,  299,  299,  299,  299,  299,
      299,  299,  299,  299,  299,  299,   97,   97,   98,  117,
      333,  333,  335,  335,   99,  117,  335,  311,  335,  335,
      333,  100,  312,  312,  335,  102,  310,  310,  312,  312,
      118,  117,  335,  335,  335,  335,  118,  310,  310,  310,
      310,  310,  310,  313,  324,  324,  335,  335,  335,  335,
      335,  335,  118,  335,  324,  314,  314,  335,  335,  335,
      335,  335,  335,  335,  117,  314,  314,  314,  314,  314,
      314,  314,   97,   97,  144,  117,  117,  332,  332,  117,

      145,  335,  335,  335,  335,  118,  335,  146,  326,  326,
      335,  148,  324,  324,  117,  117,  118,  118,  326,  117,
      118,  335,  324,  330,  335,  330,  335,  326,  326,  335,
      117,  335,  334,  334,  335,  118,  118,  326,  335,  335,
      118,  335,  334,  334,  334,  335,  335,  335,  335,  335,
      335,  118,  335,  334,   71,  335,  335,   71,  335,  335,
      335,   71,   71,   71,   84,   84,   84,  335,  335,  335,
      335,  335,   84,  335,   84,  335,  335,   84,   84,   84,
       94,  335,   94,   94,   94,  101,  101,  101,  335,  335,
      335,  335,  101,  101,  101,  335,  335,  335,  101,  101,

      101,  116,  116,  116,  116,  116,  116,  116,  116,  116,
      116,  116,  116,  116,  116,  168,  168,  335,  168,  168,
      168,  168,  168,  168,  168,  168,  168,  168,  168,  168,
      168,  168,  169,  169,  169,  170,  335,  335,  335,  170,
      170,  170,  177,  335,  335,  177,  177,  177,  177,  182,
      182,  182,  335,  182,  335,  335,  335,  335,  335,  182,
      335,  335,  182,  182,  182,  197,  197,  335,  197,  197,
      197,  197,  197,  197,  197,  197,  197,  197,  197,  197,
      197,  197,  211,  335,  335,  335,  335,  211,  335,  335,
      335,  335,  211,  230,  335,  335,  230,  230,  230,  230,

      235,  335,  335,  235,  235,  335,  335,  335,  235,  235,
      250,  250,  250,  251,  251,  251,  335,  335,  335,  335,
      251,  251,  251,  335,  335,  335,  251,  251,  251,  288,
      335,  335,  288,  288,  288,  288,  305,  335,  335,  335,
      305,  335,  335,  335,  305,  305,    7,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335

    } ;

static yyconst short int yy_chk[1796] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    5,    6,   88,   88,

        5,    6,   14,   14,   14,   14,   14,   14,   14,   14,
       14,   14,   14,   14,   14,   14,   14,   14,   14,   14,
       14,   14,   14,   14,   14,   14,   14,   14,   14,   14,
       14,   14,   14,   14,   14,   14,   14,   14,   14,   14,
       14,   14,   14,   14,   14,   14,   14,   14,   14,   14,
       20,   20,   20,   31,   22,   77,   39,  323,   32,   77,
       23,   31,   32,   31,   22,   20,   22,   22,   32,   33,
       23,   20,   23,   23,  174,  174,   22,   39,   20,   33,
       33,   22,   23,   29,   29,   29,   34,   23,  237,   34,
      237,   29,  280,  280,   40,   22,   34,   34,   29,   43,

       35,   34,   29,   30,   30,   30,   36,   37,   45,   36,
       35,   30,   35,   37,   38,   40,   36,   37,   30,   46,
       43,   61,   30,  241,   38,   38,   65,   63,  317,   45,
       90,   61,   30,   63,   65,   90,   90,  319,   65,  319,
       46,  241,   30,   42,   42,   42,   42,   42,   42,   42,
       42,   42,   42,   42,   42,   42,   42,   42,   42,   42,
       42,   42,   42,   42,   42,   42,   42,   42,   42,   42,
       42,   42,   42,   42,   42,   42,   42,   42,   42,   42,
       42,   42,   42,   42,   42,   42,   42,   42,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   44,   44,   44,

       44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,   44,   44,   47,   48,   49,  101,
      103,  105,  104,   47,  104,  106,  107,  310,   49,  101,
      103,  105,  104,  103,  105,  106,  107,   47,   48,   49,
       50,   50,   50,   50,   50,   50,   50,   50,   50,   50,
       50,   50,   50,   50,   50,   50,   50,   50,   50,   50,
       50,   50,   50,   50,   50,   50,   50,   50,   50,   50,
       50,   50,   50,   50,   50,   50,   50,   50,   50,   50,

       50,   50,   50,   50,   50,   50,   50,   51,   97,   97,
       97,   58,  110,   54,   56,  278,   59,  116,   57,  309,
      278,  278,  110,   97,  118,   51,  110,   97,   51,   52,
       52,   54,   58,   56,   54,   56,   57,   59,  116,   57,
       52,   89,   52,   52,  125,  118,  112,   98,   98,   98,
       52,   89,   52,   89,   89,   98,  112,   52,  308,  108,
      172,  108,   98,   89,  305,  125,   98,  304,   89,  108,
      172,   52,   55,   55,   55,   55,   55,   55,   55,   55,
       55,   55,   55,   55,   55,   55,   55,   55,   55,   55,
       55,   55,   55,   55,   55,   55,   55,   55,   55,   55,

       55,   55,   55,   55,   55,   55,   55,   55,   55,   55,
       55,   55,   55,   55,   55,   55,   55,   60,   60,   60,
       60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
       60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
       60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
       60,   60,   60,   60,   60,   60,   60,   60,   60,   60,
       60,   60,   60,   60,   70,   70,   70,   70,   70,   70,
       70,   70,   70,  301,   70,   70,   70,   70,   70,   70,
       70,   70,   70,   70,   70,   70,   70,   70,   70,   70,
       70,   70,   70,   70,   70,   70,   70,   70,   70,   70,

       70,   70,   70,   70,   70,   70,   70,   70,   70,   70,
       70,   70,   72,   72,   72,  100,  100,  100,  109,  100,
       72,  111,  298,  113,  292,  175,  175,  114,  109,  111,
      114,  111,  113,  113,  109,  175,  100,  114,  302,  302,
       72,   72,   72,   72,   72,   72,   72,   72,   72,   72,
       72,   72,   72,   72,   72,   72,   72,   72,   72,   78,
      115,  173,  173,  184,  185,  186,  291,  187,  186,  290,
      115,  173,  115,  184,  185,  186,  173,  187,  288,   78,
       78,   78,   78,   78,   78,   78,   78,   78,   78,   78,
       78,   78,   78,   78,   78,   78,   78,   78,   81,   81,

       81,  128,  132,  134,  126,  126,  287,  240,  188,  126,
      138,  139,  286,   81,  240,  141,  134,  134,  188,   81,
      142,  145,  128,  132,  134,  136,   81,  121,  121,  121,
      126,  138,  139,  136,  189,  121,  141,  190,  136,  136,
      191,  142,  145,  192,  189,  190,  136,  190,  328,  328,
      191,  285,  284,  192,  283,  121,  121,  121,  121,  121,
      121,  121,  121,  121,  121,  121,  121,  121,  121,  121,
      121,  121,  121,  121,  124,  124,  124,  124,  124,  124,
      124,  124,  124,  124,  124,  124,  124,  124,  124,  124,
      124,  124,  124,  129,  282,  279,  277,  270,  129,  129,

      129,  129,  129,  129,  129,  129,  129,  129,  129,  129,
      129,  129,  129,  129,  129,  129,  129,  129,  130,  130,
      344,  345,  354,  269,  257,  344,  345,  354,  130,  130,
      130,  130,  130,  130,  130,  130,  130,  130,  130,  130,
      130,  130,  130,  130,  130,  130,  130,  130,  133,  148,
      193,  355,  163,  163,  163,  250,  355,  249,  235,  216,
      193,  133,  133,  242,  148,  164,  164,  164,  235,  133,
      148,  163,  133,  133,  133,  133,  133,  133,  137,  137,
      216,  242,  248,  247,  164,  246,  245,  239,  233,  137,
      137,  137,  137,  137,  137,  140,  140,  140,  140,  140,

      140,  140,  140,  140,  140,  140,  140,  140,  140,  140,
      140,  140,  140,  140,  140,  140,  140,  140,  140,  140,
      140,  140,  140,  140,  140,  140,  140,  140,  140,  140,
      140,  140,  140,  140,  140,  140,  140,  140,  140,  140,
      144,  144,  144,  230,  144,  178,  178,  178,  144,  228,
      225,  238,  238,  178,  223,  144,  219,  210,  202,  144,
      178,  238,  177,  171,  178,  144,  146,  146,  146,  146,
      146,  146,  146,  146,  146,  146,  146,  146,  146,  146,
      146,  146,  146,  146,  146,  146,  146,  146,  146,  146,
      146,  146,  146,  146,  146,  146,  146,  146,  146,  146,

      146,  146,  146,  146,  146,  146,  146,  146,  146,  146,
      146,  146,  146,  166,  166,  179,  179,  179,  169,  179,
      167,  227,  227,  227,  166,  166,  166,  166,  166,  166,
      182,  182,  182,  244,  197,  197,  179,  180,  182,  197,
      227,  180,  180,  244,  180,  182,  180,  162,  161,  182,
      160,  180,  180,  180,  243,  194,  194,  194,  180,  194,
      197,  199,  199,  252,  159,  204,  199,  157,  180,  180,
      180,  180,  243,  252,  194,  253,  180,  205,  204,  204,
      194,  195,  236,  236,  156,  253,  204,  199,  204,  254,
      205,  205,  236,  204,  195,  195,  155,  236,  205,  254,

      154,  151,  150,  149,  143,  195,  195,  195,  195,  195,
      195,  200,  200,  200,  200,  200,  200,  200,  200,  200,
      200,  200,  200,  200,  200,  200,  200,  200,  200,  200,
      200,  200,  200,  200,  200,  200,  200,  200,  200,  200,
      200,  200,  200,  200,  200,  200,  200,  200,  200,  200,
      200,  200,  200,  200,  200,  200,  203,  327,  327,  327,
      135,  303,  303,  131,  127,  123,  203,  261,  203,  203,
      289,  303,  206,  255,  255,  255,  327,  255,  203,  203,
      203,  203,  203,  203,  203,  206,  206,  120,  261,  306,
      306,  289,  255,  206,  102,  206,  207,  207,  255,  306,

       94,  256,  263,  313,   85,   76,   75,  207,  207,  207,
      207,  207,  207,  209,  209,  209,  256,  209,  251,  251,
      251,  209,  256,  263,  313,   74,  251,   73,  209,   69,
      263,  313,  209,  251,   68,   67,   66,  251,  209,  214,
      214,  214,  214,  214,  214,  214,  214,  214,  214,  214,
      214,  214,  214,  214,  214,  214,  214,  214,  214,  215,
      215,   64,   62,   53,   27,   26,   25,   24,   21,  215,
      215,  215,  215,  215,  215,  215,  215,  215,  215,  215,
      215,  215,  215,  215,  215,  215,  215,  215,  215,  229,
      229,   19,   17,    7,    0,  296,    0,    0,    0,    0,

      229,  229,  229,  229,  229,  229,  258,  258,  296,  296,
        0,  266,    0,    0,    0,    0,  296,  258,  258,  258,
      258,  258,  258,  259,  266,  266,    0,    0,  318,  318,
      320,  320,  266,    0,  266,  267,  259,  259,  318,  266,
      320,    0,    0,  267,    0,  267,    0,  259,  259,  259,
      259,  259,  259,  264,  264,    0,  267,  268,    0,    0,
        0,    0,  331,  331,  264,  264,  264,  264,  264,  264,
      268,  268,  331,    0,    0,    0,    0,    0,  268,    0,
      268,  271,  271,  271,  271,  271,  271,  271,  271,  271,
      271,  271,  271,  271,  271,  271,  271,  271,  271,  271,

      271,  271,  271,  271,  271,  271,  271,  271,  271,  271,
      271,  271,  271,  271,  271,  271,  271,  271,  271,  271,
      271,  271,  271,  271,  271,  271,  281,  281,  281,  294,
      333,  333,    0,    0,  281,  311,    0,  294,    0,    0,
      333,  281,  294,  294,    0,  281,  293,  293,  311,  311,
      294,  312,    0,    0,    0,    0,  311,  293,  293,  293,
      293,  293,  293,  295,  312,  312,    0,    0,    0,    0,
        0,    0,  312,    0,  312,  295,  295,    0,    0,    0,
        0,    0,    0,    0,  330,  295,  295,  295,  295,  295,
      295,  295,  300,  300,  300,  315,  300,  330,  330,  324,

      300,    0,    0,    0,    0,  330,    0,  300,  315,  315,
        0,  300,  324,  324,  326,  325,  315,  300,  315,  332,
      324,    0,  324,  325,    0,  325,    0,  326,  326,    0,
      334,    0,  332,  332,    0,  326,  325,  326,    0,    0,
      332,    0,  332,  334,  334,    0,    0,    0,    0,    0,
        0,  334,    0,  334,  336,    0,    0,  336,    0,    0,
        0,  336,  336,  336,  337,  337,  337,    0,    0,    0,
        0,    0,  337,    0,  337,    0,    0,  337,  337,  337,
      338,    0,  338,  338,  338,  339,  339,  339,    0,    0,
        0,    0,  339,  339,  339,    0,    0,    0,  339,  339,

      339,  340,  340,  340,  340,  340,  340,  340,  340,  340,
      340,  340,  340,  340,  340,  341,  341,    0,  341,  341,
      341,  341,  341,  341,  341,  341,  341,  341,  341,  341,
      341,  341,  342,  342,  342,  343,    0,    0,    0,  343,
      343,  343,  346,    0,    0,  346,  346,  346,  346,  347,
      347,  347,    0,  347,    0,    0,    0,    0,    0,  347,
        0,    0,  347,  347,  347,  348,  348,    0,  348,  348,
      348,  348,  348,  348,  348,  348,  348,  348,  348,  348,
      348,  348,  349,    0,    0,    0,    0,  349,    0,    0,
        0,    0,  349,  350,    0,    0,  350,  350,  350,  350,

      351,    0,    0,  351,  351,    0,    0,    0,  351,  351,
      352,  352,  352,  353,  353,  353,    0,    0,    0,    0,
      353,  353,  353,    0,    0,    0,  353,  353,  353,  356,
        0,    0,  356,  356,  356,  356,  357,    0,    0,    0,
      357,    0,    0,    0,  357,  357,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335,  335,  335,  335,  335,  335,
      335,  335,  335,  335,  335

    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
#line 1 "ada-lex.l"
#define INITIAL 0
/* FLEX lexer for Ada expressions, for GDB.
   Copyright (C) 1994, 1997, 1998, 2000, 2001, 2002, 2003
   Free Software Foundation, Inc.

This file is part of GDB.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
/*----------------------------------------------------------------------*/
/* The converted version of this file is to be included in ada-exp.y, */
/* the Ada parser for gdb.  The function yylex obtains characters from */
/* the global pointer lexptr.  It returns a syntactic category for */
/* each successive token and places a semantic value into yylval */
/* (ada-lval), defined by the parser.   */
#line 45 "ada-lex.l"

#define NUMERAL_WIDTH 256
#define LONGEST_SIGN ((ULONGEST) 1 << (sizeof(LONGEST) * HOST_CHAR_BIT - 1))

/* Temporary staging for numeric literals.  */
static char numbuf[NUMERAL_WIDTH];
 static void canonicalizeNumeral (char *s1, const char *);
static int processInt (const char *, const char *, const char *);
static int processReal (const char *);
static int processId (const char *, int);
static int processAttribute (const char *);
static int find_dot_all (const char *);

#undef YY_DECL
#define YY_DECL static int yylex ( void )

#undef YY_INPUT
#define YY_INPUT(BUF, RESULT, MAX_SIZE) \
    if ( *lexptr == '\000' ) \
      (RESULT) = YY_NULL; \
    else \
      { \
        *(BUF) = *lexptr; \
        (RESULT) = 1; \
	lexptr += 1; \
      }

static char *tempbuf = NULL;
static int tempbufsize = 0;
static int tempbuf_len;
static struct block *left_block_context;

static void resize_tempbuf (unsigned int);

static void block_lookup (char *, char *);

static int name_lookup (char *, char *, int *, int);

static int find_dot_all (const char *);

#define IN_STRING 1
#define BEFORE_QUAL_QUOTE 2

#line 943 "ada-lex.c"

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap YY_PROTO(( void ));
#else
extern int yywrap YY_PROTO(( void ));
#endif
#endif

#ifndef YY_NO_UNPUT
static void yyunput YY_PROTO(( int c, char *buf_ptr ));
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy YY_PROTO(( char *, yyconst char *, int ));
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen YY_PROTO(( yyconst char * ));
#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput YY_PROTO(( void ));
#else
static int input YY_PROTO(( void ));
#endif
#endif

#if YY_STACK_USED
static int yy_start_stack_ptr = 0;
static int yy_start_stack_depth = 0;
static int *yy_start_stack = 0;
#ifndef YY_NO_PUSH_STATE
static void yy_push_state YY_PROTO(( int new_state ));
#endif
#ifndef YY_NO_POP_STATE
static void yy_pop_state YY_PROTO(( void ));
#endif
#ifndef YY_NO_TOP_STATE
static int yy_top_state YY_PROTO(( void ));
#endif

#else
#define YY_NO_PUSH_STATE 1
#define YY_NO_POP_STATE 1
#define YY_NO_TOP_STATE 1
#endif

#ifdef YY_MALLOC_DECL
YY_MALLOC_DECL
#else
#if __STDC__
#ifndef __cplusplus
#include <stdlib.h>
#endif
#else
/* Just try to get by without declaring the routines.  This will fail
 * miserably on non-ANSI systems for which sizeof(size_t) != sizeof(int)
 * or sizeof(void*) != sizeof(int).
 */
#endif
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( yy_current_buffer->yy_is_interactive ) \
		{ \
		int c = '*', n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else if ( ((result = fread( buf, 1, max_size, yyin )) == 0) \
		  && ferror( yyin ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL int yylex YY_PROTO(( void ))
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

YY_DECL
	{
	register yy_state_type yy_current_state;
	register char *yy_cp = NULL, *yy_bp = NULL;
	register int yy_act;

#line 91 "ada-lex.l"


#line 1097 "ada-lex.c"

	if ( yy_init )
		{
		yy_init = 0;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yy_start )
			yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! yy_current_buffer )
			yy_current_buffer =
				yy_create_buffer( yyin, YY_BUF_SIZE );

		yy_load_buffer_state();
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start;
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				yy_last_accepting_state = yy_current_state;
				yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 336 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 1747 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;


do_action:	/* This label is used only to access EOF actions. */


		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yy_hold_char;
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 93 "ada-lex.l"
{ }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 95 "ada-lex.l"
{ yyterminate(); }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 97 "ada-lex.l"
{
		   canonicalizeNumeral (numbuf, yytext);
		   return processInt (NULL, numbuf, strrchr(numbuf, 'e')+1);
		 }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 102 "ada-lex.l"
{
		   canonicalizeNumeral (numbuf, yytext);
		   return processInt (NULL, numbuf, NULL);
		 }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 107 "ada-lex.l"
{
		   canonicalizeNumeral (numbuf, yytext);
    		   return processInt (numbuf,
				      strchr (numbuf, '#') + 1,
				      strrchr(numbuf, '#') + 1);
		 }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 114 "ada-lex.l"
{
		   canonicalizeNumeral (numbuf, yytext);
    		   return processInt (numbuf, strchr (numbuf, '#') + 1, NULL);
		 }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 119 "ada-lex.l"
{
		  canonicalizeNumeral (numbuf, yytext+2);
		  return processInt ("16#", numbuf, NULL);
		}
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 125 "ada-lex.l"
{
		   canonicalizeNumeral (numbuf, yytext);
		   return processReal (numbuf);
		}
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 130 "ada-lex.l"
{
		   canonicalizeNumeral (numbuf, yytext);
		   return processReal (numbuf);
		}
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 135 "ada-lex.l"
{
                   error ("Based real literals not implemented yet.");
		}
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 139 "ada-lex.l"
{
                   error ("Based real literals not implemented yet.");
		}
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 143 "ada-lex.l"
{
		   yylval.typed_val.type = type_char ();
		   yylval.typed_val.val = yytext[1];
		   return CHARLIT;
		}
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 149 "ada-lex.l"
{
                   int v;
                   yylval.typed_val.type = type_char ();
		   sscanf (yytext+3, "%2x", &v);
		   yylval.typed_val.val = v;
		   return CHARLIT;
		}
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 157 "ada-lex.l"
{
		   tempbuf_len = 0;
		   BEGIN IN_STRING;
		}
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 162 "ada-lex.l"
{
		   resize_tempbuf (yyleng+tempbuf_len);
		   strncpy (tempbuf+tempbuf_len, yytext, yyleng-1);
		   tempbuf_len += yyleng-1;
		   yylval.sval.ptr = tempbuf;
		   yylval.sval.length = tempbuf_len;
		   BEGIN INITIAL;
		   return STRING;
		}
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 172 "ada-lex.l"
{
		   int n;
		   resize_tempbuf (yyleng-5+tempbuf_len+1);
		   strncpy (tempbuf+tempbuf_len, yytext, yyleng-6);
		   sscanf(yytext+yyleng-4, "%2x", &n);
		   tempbuf[yyleng-6+tempbuf_len] = (char) n;
		   tempbuf_len += yyleng-5;
		}
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 181 "ada-lex.l"
{
		   int n;
		   resize_tempbuf (yyleng-4+tempbuf_len+1);
		   strncpy (tempbuf+tempbuf_len, yytext, yyleng-6);
		   tempbuf[yyleng-5+tempbuf_len] = '"';
		   tempbuf_len += yyleng-4;
		}
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 189 "ada-lex.l"
{
		  while (*lexptr != 'i' && *lexptr != 'I')
		    lexptr -= 1;
		  yyrestart(NULL);
		  return 0;
		}
	YY_BREAK
/* ADA KEYWORDS */
case 19:
YY_RULE_SETUP
#line 198 "ada-lex.l"
{ return ABS; }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 199 "ada-lex.l"
{ return _AND_; }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 200 "ada-lex.l"
{ return ELSE; }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 201 "ada-lex.l"
{ return IN; }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 202 "ada-lex.l"
{ return MOD; }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 203 "ada-lex.l"
{ return NEW; }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 204 "ada-lex.l"
{ return NOT; }
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 205 "ada-lex.l"
{ return NULL_PTR; }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 206 "ada-lex.l"
{ return OR; }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 207 "ada-lex.l"
{ return REM; }
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 208 "ada-lex.l"
{ return THEN; }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 209 "ada-lex.l"
{ return XOR; }
	YY_BREAK
/* ATTRIBUTES */
case 31:
YY_RULE_SETUP
#line 213 "ada-lex.l"
{ return processAttribute (yytext+1); }
	YY_BREAK
/* PUNCTUATION */
case 32:
YY_RULE_SETUP
#line 217 "ada-lex.l"
{ return ARROW; }
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 218 "ada-lex.l"
{ return DOTDOT; }
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 219 "ada-lex.l"
{ return STARSTAR; }
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 220 "ada-lex.l"
{ return ASSIGN; }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 221 "ada-lex.l"
{ return NOTEQUAL; }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 222 "ada-lex.l"
{ return LEQ; }
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 223 "ada-lex.l"
{ return GEQ; }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 225 "ada-lex.l"
{ BEGIN INITIAL; return '\''; }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 227 "ada-lex.l"
{ return yytext[0]; }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 229 "ada-lex.l"
{ if (paren_depth == 0 && comma_terminates)
		    {
		      lexptr -= 1;
		      yyrestart(NULL);
		      return 0;
		    }
		  else
		    return ',';
		}
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 239 "ada-lex.l"
{ paren_depth += 1; return '('; }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 240 "ada-lex.l"
{ if (paren_depth == 0)
		    {
		      lexptr -= 1;
		      yyrestart(NULL);
		      return 0;
		    }
		  else
 		    {
		      paren_depth -= 1;
		      return ')';
		    }
		}
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 253 "ada-lex.l"
{ return DOT_ALL; }
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 255 "ada-lex.l"
{
	 	  processId (yytext+1, yyleng-1);
	          return DOT_ID;
		}
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 260 "ada-lex.l"
{
                  int all_posn = find_dot_all (yytext);
		  int token_type, segments, k;
		  int quote_follows;

                  if (all_posn == -1 && yytext[yyleng-1] == '\'')
		    {
		      quote_follows = 1;
		      do {
			yyless (yyleng-1);
		      } while (yytext[yyleng-1] == ' ');
		    }
		  else
		    quote_follows = 0;

                  if (all_posn >= 0)
		    yyless (all_posn);
                  processId(yytext, yyleng);
                  segments = name_lookup (ada_encode (yylval.ssym.stoken.ptr),
		                          yylval.ssym.stoken.ptr, 
                                          &token_type,
					  MAX_RENAMING_CHAIN_LENGTH);
		  left_block_context = NULL;
		  for (k = yyleng; segments > 0 && k > 0; k -= 1)
                    {
		      if (yytext[k-1] == '.')
			segments -= 1;
		      quote_follows = 0;
		    }
		  if (k <= 0)
		    error ("confused by name %s", yytext);
		  yyless (k);
		  if (quote_follows)
		    BEGIN BEFORE_QUAL_QUOTE;
		  return token_type;
                }
	YY_BREAK
/* GDB EXPRESSION CONSTRUCTS  */
case 47:
YY_RULE_SETUP
#line 300 "ada-lex.l"
{
                  processId(yytext, yyleng-2);
                  block_lookup (yylval.ssym.stoken.ptr, yylval.ssym.stoken.ptr);
                  return BLOCKNAME;
		}
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 306 "ada-lex.l"
{
                  processId(yytext, yyleng-2);
                  block_lookup (ada_encode (yylval.ssym.stoken.ptr),
                                yylval.ssym.stoken.ptr);
                  return BLOCKNAME;
		}
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 313 "ada-lex.l"
{ return yytext[0]; }
	YY_BREAK
/* REGISTERS AND GDB CONVENIENCE VARIABLES */
case 50:
YY_RULE_SETUP
#line 317 "ada-lex.l"
{
		  yylval.sval.ptr = yytext;
		  yylval.sval.length = yyleng;
		  return SPECIAL_VARIABLE;
		}
	YY_BREAK
/* CATCH-ALL ERROR CASE */
case 51:
YY_RULE_SETUP
#line 325 "ada-lex.l"
{ error ("Invalid character '%s' in expression.", yytext); }
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 326 "ada-lex.l"
YY_FATAL_ERROR( "flex scanner jammed" );
	YY_BREAK
#line 1582 "ada-lex.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(IN_STRING):
case YY_STATE_EOF(BEFORE_QUAL_QUOTE):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between yy_current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yy_n_chars = yy_current_buffer->yy_n_chars;
			yy_current_buffer->yy_input_file = yyin;
			yy_current_buffer->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yy_c_buf_p <= &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yy_c_buf_p = yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state();

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer() )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yy_did_buffer_switch_on_eof = 0;

				if ( yywrap() )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yy_c_buf_p =
					yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yy_c_buf_p =
				&yy_current_buffer->yy_ch_buf[yy_n_chars];

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of yylex */


/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

static int yy_get_next_buffer()
	{
	register char *dest = yy_current_buffer->yy_ch_buf;
	register char *source = yytext_ptr;
	register int number_to_move, i;
	int ret_val;

	if ( yy_c_buf_p > &yy_current_buffer->yy_ch_buf[yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( yy_current_buffer->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yy_c_buf_p - yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yy_c_buf_p - yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		yy_current_buffer->yy_n_chars = yy_n_chars = 0;

	else
		{
		int num_to_read =
			yy_current_buffer->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = yy_current_buffer;

			int yy_c_buf_p_offset =
				(int) (yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yyxrealloc( (void *) b->yy_ch_buf,
							 b->yy_buf_size + 2 );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = yy_current_buffer->yy_buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&yy_current_buffer->yy_ch_buf[number_to_move]),
			yy_n_chars, num_to_read );

		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	if ( yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			yy_current_buffer->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	yy_n_chars += number_to_move;
	yy_current_buffer->yy_ch_buf[yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	yy_current_buffer->yy_ch_buf[yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yytext_ptr = &yy_current_buffer->yy_ch_buf[0];

	return ret_val;
	}


/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state()
	{
	register yy_state_type yy_current_state;
	register char *yy_cp;

	yy_current_state = yy_start;

	for ( yy_cp = yytext_ptr + YY_MORE_ADJ; yy_cp < yy_c_buf_p; ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yy_last_accepting_state = yy_current_state;
			yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 336 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
	}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */

#ifdef YY_USE_PROTOS
static yy_state_type yy_try_NUL_trans( yy_state_type yy_current_state )
#else
static yy_state_type yy_try_NUL_trans( yy_current_state )
yy_state_type yy_current_state;
#endif
	{
	register int yy_is_jam;
	register char *yy_cp = yy_c_buf_p;

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yy_last_accepting_state = yy_current_state;
		yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 336 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 335);

	return yy_is_jam ? 0 : yy_current_state;
	}


#ifndef YY_NO_UNPUT
#ifdef YY_USE_PROTOS
static void yyunput( int c, register char *yy_bp )
#else
static void yyunput( c, yy_bp )
int c;
register char *yy_bp;
#endif
	{
	register char *yy_cp = yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yy_hold_char;

	if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = yy_n_chars + 2;
		register char *dest = &yy_current_buffer->yy_ch_buf[
					yy_current_buffer->yy_buf_size + 2];
		register char *source =
				&yy_current_buffer->yy_ch_buf[number_to_move];

		while ( source > yy_current_buffer->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		yy_current_buffer->yy_n_chars =
			yy_n_chars = yy_current_buffer->yy_buf_size;

		if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;


	yytext_ptr = yy_bp;
	yy_hold_char = *yy_cp;
	yy_c_buf_p = yy_cp;
	}
#endif	/* ifndef YY_NO_UNPUT */


#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput()
#else
static int input()
#endif
	{
	int c;

	*yy_c_buf_p = yy_hold_char;

	if ( *yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yy_c_buf_p < &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			/* This was really a NUL. */
			*yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = yy_c_buf_p - yytext_ptr;
			++yy_c_buf_p;

			switch ( yy_get_next_buffer() )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin );

					/* fall through */

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap() )
						return EOF;

					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yy_c_buf_p = yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yy_c_buf_p;	/* cast for 8-bit char's */
	*yy_c_buf_p = '\0';	/* preserve yytext */
	yy_hold_char = *++yy_c_buf_p;


	return c;
	}
#endif /* YY_NO_INPUT */

#ifdef YY_USE_PROTOS
void yyrestart( FILE *input_file )
#else
void yyrestart( input_file )
FILE *input_file;
#endif
	{
	if ( ! yy_current_buffer )
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE );

	yy_init_buffer( yy_current_buffer, input_file );
	yy_load_buffer_state();
	}


#ifdef YY_USE_PROTOS
void yy_switch_to_buffer( YY_BUFFER_STATE new_buffer )
#else
void yy_switch_to_buffer( new_buffer )
YY_BUFFER_STATE new_buffer;
#endif
	{
	if ( yy_current_buffer == new_buffer )
		return;

	if ( yy_current_buffer )
		{
		/* Flush out information for old buffer. */
		*yy_c_buf_p = yy_hold_char;
		yy_current_buffer->yy_buf_pos = yy_c_buf_p;
		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	yy_current_buffer = new_buffer;
	yy_load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yy_did_buffer_switch_on_eof = 1;
	}


#ifdef YY_USE_PROTOS
void yy_load_buffer_state( void )
#else
void yy_load_buffer_state()
#endif
	{
	yy_n_chars = yy_current_buffer->yy_n_chars;
	yytext_ptr = yy_c_buf_p = yy_current_buffer->yy_buf_pos;
	yyin = yy_current_buffer->yy_input_file;
	yy_hold_char = *yy_c_buf_p;
	}


#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_create_buffer( FILE *file, int size )
#else
YY_BUFFER_STATE yy_create_buffer( file, size )
FILE *file;
int size;
#endif
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file );

	return b;
	}


#ifdef YY_USE_PROTOS
void yy_delete_buffer( YY_BUFFER_STATE b )
#else
void yy_delete_buffer( b )
YY_BUFFER_STATE b;
#endif
	{
	if ( ! b )
		return;

	if ( b == yy_current_buffer )
		yy_current_buffer = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yy_flex_free( (void *) b->yy_ch_buf );

	yy_flex_free( (void *) b );
	}



#ifdef YY_USE_PROTOS
void yy_init_buffer( YY_BUFFER_STATE b, FILE *file )
#else
void yy_init_buffer( b, file )
YY_BUFFER_STATE b;
FILE *file;
#endif


	{
	yy_flush_buffer( b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

#if YY_ALWAYS_INTERACTIVE
	b->yy_is_interactive = 1;
#else
#if YY_NEVER_INTERACTIVE
	b->yy_is_interactive = 0;
#else
	b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
#endif
#endif
	}


#ifdef YY_USE_PROTOS
void yy_flush_buffer( YY_BUFFER_STATE b )
#else
void yy_flush_buffer( b )
YY_BUFFER_STATE b;
#endif

	{
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == yy_current_buffer )
		yy_load_buffer_state();
	}


#ifndef YY_NO_SCAN_BUFFER
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_buffer( char *base, yy_size_t size )
#else
YY_BUFFER_STATE yy_scan_buffer( base, size )
char *base;
yy_size_t size;
#endif
	{
	YY_BUFFER_STATE b;

	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b );

	return b;
	}
#endif


#ifndef YY_NO_SCAN_STRING
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_string( yyconst char *yy_str )
#else
YY_BUFFER_STATE yy_scan_string( yy_str )
yyconst char *yy_str;
#endif
	{
	int len;
	for ( len = 0; yy_str[len]; ++len )
		;

	return yy_scan_bytes( yy_str, len );
	}
#endif


#ifndef YY_NO_SCAN_BYTES
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_bytes( yyconst char *bytes, int len )
#else
YY_BUFFER_STATE yy_scan_bytes( bytes, len )
yyconst char *bytes;
int len;
#endif
	{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;

	/* Get memory for full buffer, including space for trailing EOB's. */
	n = len + 2;
	buf = (char *) yy_flex_alloc( n );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < len; ++i )
		buf[i] = bytes[i];

	buf[len] = buf[len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
	}
#endif


#ifndef YY_NO_PUSH_STATE
#ifdef YY_USE_PROTOS
static void yy_push_state( int new_state )
#else
static void yy_push_state( new_state )
int new_state;
#endif
	{
	if ( yy_start_stack_ptr >= yy_start_stack_depth )
		{
		yy_size_t new_size;

		yy_start_stack_depth += YY_START_STACK_INCR;
		new_size = yy_start_stack_depth * sizeof( int );

		if ( ! yy_start_stack )
			yy_start_stack = (int *) yy_flex_alloc( new_size );

		else
			yy_start_stack = (int *) yyxrealloc(
					(void *) yy_start_stack, new_size );

		if ( ! yy_start_stack )
			YY_FATAL_ERROR(
			"out of memory expanding start-condition stack" );
		}

	yy_start_stack[yy_start_stack_ptr++] = YY_START;

	BEGIN(new_state);
	}
#endif


#ifndef YY_NO_POP_STATE
static void yy_pop_state()
	{
	if ( --yy_start_stack_ptr < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(yy_start_stack[yy_start_stack_ptr]);
	}
#endif


#ifndef YY_NO_TOP_STATE
static int yy_top_state()
	{
	return yy_start_stack[yy_start_stack_ptr - 1];
	}
#endif

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

#ifdef YY_USE_PROTOS
static void yy_fatal_error( yyconst char msg[] )
#else
static void yy_fatal_error( msg )
char msg[];
#endif
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
	}



/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = yy_hold_char; \
		yy_c_buf_p = yytext + n; \
		yy_hold_char = *yy_c_buf_p; \
		*yy_c_buf_p = '\0'; \
		yyleng = n; \
		} \
	while ( 0 )


/* Internal utility routines. */

#ifndef yytext_ptr
#ifdef YY_USE_PROTOS
static void yy_flex_strncpy( char *s1, yyconst char *s2, int n )
#else
static void yy_flex_strncpy( s1, s2, n )
char *s1;
yyconst char *s2;
int n;
#endif
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif

#ifdef YY_NEED_STRLEN
#ifdef YY_USE_PROTOS
static int yy_flex_strlen( yyconst char *s )
#else
static int yy_flex_strlen( s )
yyconst char *s;
#endif
	{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
	}
#endif


#ifdef YY_USE_PROTOS
static void *yy_flex_alloc( yy_size_t size )
#else
static void *yy_flex_alloc( size )
yy_size_t size;
#endif
	{
	return (void *) xmalloc( size );
	}

#ifdef YY_USE_PROTOS
static void *yyxrealloc( void *ptr, yy_size_t size )
#else
static void *yyxrealloc( ptr, size )
void *ptr;
yy_size_t size;
#endif
	{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) xrealloc( (char *) ptr, size );
	}

#ifdef YY_USE_PROTOS
static void yy_flex_free( void *ptr )
#else
static void yy_flex_free( ptr )
void *ptr;
#endif
	{
	free( ptr );
	}

#if YY_MAIN
int main()
	{
	yylex();
	return 0;
	}
#endif
#line 326 "ada-lex.l"


#include <ctype.h>
#include "gdb_string.h"

/* Initialize the lexer for processing new expression */
void
lexer_init (FILE *inp)
{
  BEGIN INITIAL;
  yyrestart (inp);
}


/* Make sure that tempbuf points at an array at least N characters long.  */

static void
resize_tempbuf (unsigned int n)
{
  if (tempbufsize < n)
    {
      tempbufsize = (n+63) & ~63;
      tempbuf = xrealloc (tempbuf, tempbufsize);
    }
}

/* Copy S2 to S1, removing all underscores, and downcasing all letters.  */

static void
canonicalizeNumeral (char *s1, const char *s2)
{
  for (; *s2 != '\000'; s2 += 1)
    {
      if (*s2 != '_')
	{
	  *s1 = tolower(*s2);
	  s1 += 1;
	}
    }
  s1[0] = '\000';
}

#define HIGH_BYTE_POSN ((sizeof (ULONGEST) - 1) * HOST_CHAR_BIT)

/* True (non-zero) iff DIGIT is a valid digit in radix BASE,
   where 2 <= BASE <= 16.  */

static int
is_digit_in_base (unsigned char digit, int base)
{
  if (!isxdigit (digit))
    return 0;
  if (base <= 10)
    return (isdigit (digit) && digit < base + '0');
  else
    return (isdigit (digit) || tolower (digit) < base - 10 + 'a');
}

static int
digit_to_int (unsigned char c)
{
  if (isdigit (c))
    return c - '0';
  else
    return tolower (c) - 'a' + 10;
}

/* As for strtoul, but for ULONGEST results.  */
ULONGEST
strtoulst (const char *num, const char **trailer, int base)
{
  unsigned int high_part;
  ULONGEST result;
  int i;
  unsigned char lim;

  if (base < 2 || base > 16)
    {
      errno = EINVAL;
      return 0;
    }
  lim = base - 1 + '0';

  result = high_part = 0;
  for (i = 0; is_digit_in_base (num[i], base); i += 1)
    {
      result = result*base + digit_to_int (num[i]);
      high_part = high_part*base + (unsigned int) (result >> HIGH_BYTE_POSN);
      result &= ((ULONGEST) 1 << HIGH_BYTE_POSN) - 1;
      if (high_part > 0xff)
	{
	  errno = ERANGE;
	  result = high_part = 0;
	  break;
	}
    }

  if (trailer != NULL)
    *trailer = &num[i];

  return result + ((ULONGEST) high_part << HIGH_BYTE_POSN);
}



/* Interprets the prefix of NUM that consists of digits of the given BASE
   as an integer of that BASE, with the string EXP as an exponent.
   Puts value in yylval, and returns INT, if the string is valid.  Causes
   an error if the number is improperly formated.   BASE, if NULL, defaults
   to "10", and EXP to "1".  The EXP does not contain a leading 'e' or 'E'.  */

static int
processInt (const char *base0, const char *num0, const char *exp0)
{
  ULONGEST result;
  long exp;
  int base;

  char *trailer;

  if (base0 == NULL)
    base = 10;
  else
    {
      base = strtol (base0, (char **) NULL, 10);
      if (base < 2 || base > 16)
	error ("Invalid base: %d.", base);
    }

  if (exp0 == NULL)
    exp = 0;
  else
    exp = strtol(exp0, (char **) NULL, 10);

  errno = 0;
  result = strtoulst (num0, (const char **) &trailer, base);
  if (errno == ERANGE)
    error ("Integer literal out of range");
  if (isxdigit(*trailer))
    error ("Invalid digit `%c' in based literal", *trailer);

  while (exp > 0)
    {
      if (result > (ULONG_MAX / base))
	error ("Integer literal out of range");
      result *= base;
      exp -= 1;
    }

  if ((result >> (TARGET_INT_BIT-1)) == 0)
    yylval.typed_val.type = type_int ();
  else if ((result >> (TARGET_LONG_BIT-1)) == 0)
    yylval.typed_val.type = type_long ();
  else if (((result >> (TARGET_LONG_BIT-1)) >> 1) == 0)
    {
      /* We have a number representable as an unsigned integer quantity.
         For consistency with the C treatment, we will treat it as an
	 anonymous modular (unsigned) quantity.  Alas, the types are such
	 that we need to store .val as a signed quantity.  Sorry
         for the mess, but C doesn't officially guarantee that a simple
         assignment does the trick (no, it doesn't; read the reference manual).
       */
      yylval.typed_val.type = builtin_type_unsigned_long;
      if (result & LONGEST_SIGN)
	yylval.typed_val.val =
	  (LONGEST) (result & ~LONGEST_SIGN)
	  - (LONGEST_SIGN>>1) - (LONGEST_SIGN>>1);
      else
	yylval.typed_val.val = (LONGEST) result;
      return INT;
    }
  else
    yylval.typed_val.type = type_long_long ();

  yylval.typed_val.val = (LONGEST) result;
  return INT;
}

#if defined (PRINTF_HAS_LONG_DOUBLE)
#  undef PRINTF_HAS_LONG_DOUBLE
#  define PRINTF_HAS_LONG_DOUBLE 1
#else
#  define PRINTF_HAS_LONG_DOUBLE 0
#endif

static int
processReal (const char *num0)
{
#if defined (PRINTF_HAS_LONG_DOUBLE)
  if (sizeof (DOUBLEST) > sizeof (double))
    sscanf (num0, "%Lg", &yylval.typed_val_float.dval);
  else
#endif
    {
      double temp;
      sscanf (num0, "%lg", &temp);
      yylval.typed_val_float.dval = temp;
    }

  yylval.typed_val_float.type = type_float ();
  if (sizeof(DOUBLEST) >= TARGET_DOUBLE_BIT / TARGET_CHAR_BIT)
    yylval.typed_val_float.type = type_double ();
  if (sizeof(DOUBLEST) >= TARGET_LONG_DOUBLE_BIT / TARGET_CHAR_BIT)
    yylval.typed_val_float.type = type_long_double ();

  return FLOAT;
}

static int
processId (const char *name0, int len)
{
  char *name = obstack_alloc (&temp_parse_space, len + 11);
  int i0, i;

  while (len > 0 && isspace (name0[len-1]))
    len -= 1;
  i = i0 = 0;
  while (i0 < len)
    {
      if (isalnum (name0[i0]))
	{
	  name[i] = tolower (name0[i0]);
	  i += 1; i0 += 1;
	}
      else switch (name0[i0])
	{
	default:
	  name[i] = name0[i0];
	  i += 1; i0 += 1;
	  break;
	case ' ': case '\t':
	  i0 += 1;
	  break;
	case '\'':
	  i0 += 1;
	  while (i0 < len && name0[i0] != '\'')
	    {
	      name[i] = name0[i0];
	      i += 1; i0 += 1;
	    }
	  i0 += 1;
	  break;
	case '<':
	  i0 += 1;
	  while (i0 < len && name0[i0] != '>')
	    {
	      name[i] = name0[i0];
	      i += 1; i0 += 1;
	    }
	  i0 += 1;
	  break;
	}
    }
  name[i] = '\000';

  yylval.ssym.sym = NULL;
  yylval.ssym.stoken.ptr = name;
  yylval.ssym.stoken.length = i;
  return NAME;
}

static void
block_lookup (char *name, char *err_name)
{
  struct ada_symbol_info *syms;
  int nsyms;
  struct symtab *symtab;
  nsyms = ada_lookup_symbol_list (name, left_block_context,
				  VAR_DOMAIN, &syms);
  if (left_block_context == NULL &&
      (nsyms == 0 || SYMBOL_CLASS (syms[0].sym) != LOC_BLOCK))
    symtab = lookup_symtab (name);
  else
    symtab = NULL;

  if (symtab != NULL)
    left_block_context = yylval.bval =
      BLOCKVECTOR_BLOCK (BLOCKVECTOR (symtab), STATIC_BLOCK);
  else if (nsyms == 0 || SYMBOL_CLASS (syms[0].sym) != LOC_BLOCK)
    {
      if (left_block_context == NULL)
	error ("No file or function \"%s\".", err_name);
      else
	error ("No function \"%s\" in specified context.", err_name);
    }
  else
    {
      left_block_context = yylval.bval = SYMBOL_BLOCK_VALUE (syms[0].sym);
      if (nsyms > 1)
	warning ("Function name \"%s\" ambiguous here", err_name);
    }
}

/* Look up NAME0 (assumed to be encoded) as a name in VAR_DOMAIN,
   setting *TOKEN_TYPE to NAME or TYPENAME, depending on what is
   found.  Try first the entire name, then the name without the last
   segment (i.e., after the last .id), etc., and return the number of
   segments that had to be removed to get a match.  Try only the full
   name if it starts with "standard__".  Calls error if no
   matches are found, using ERR_NAME in any error message.  When
   exactly one symbol match is found, it is placed in yylval.  When
   the symbol is a renaming, follow at most DEPTH steps to find the  
   ultimate definition; cause error if depth exceeded.  */

static int
name_lookup (char *name0, char *err_name, int *token_type, int depth)
{
  struct ada_symbol_info *syms;
  struct type *type;
  int len0 = strlen (name0);
  char *name = obsavestring (name0, len0, &temp_parse_space);
  int nsyms;
  int segments;

  if (depth <= 0)
    error ("Could not find renamed symbol \"%s\"", err_name);

  yylval.ssym.stoken.ptr = name;
  yylval.ssym.stoken.length = strlen (name);
  for (segments = 0; ; segments += 1)
    {
      struct type *preferred_type;
      int i, preferred_index;

      if (left_block_context == NULL)
	nsyms = ada_lookup_symbol_list (name, expression_context_block,
					VAR_DOMAIN, &syms);
      else
	nsyms = ada_lookup_symbol_list (name, left_block_context,
					VAR_DOMAIN, &syms);


      /* Check for a type renaming.  */

      if (nsyms == 1 && !ada_is_object_renaming (syms[0].sym))
        {
          struct symbol *renaming_sym =
            ada_find_renaming_symbol (SYMBOL_LINKAGE_NAME (syms[0].sym), 
				      syms[0].block);

          if (renaming_sym != NULL)
            syms[0].sym = renaming_sym;
        }

      /* Check for a type definition.  */

      /* Look for a symbol that doesn't denote void.  This is (I think) a */
      /* temporary kludge to get around problems in GNAT output.  */
      preferred_index = -1; preferred_type = NULL;
      for (i = 0; i < nsyms; i += 1)
	switch (SYMBOL_CLASS (syms[i].sym))
	  {
	  case LOC_TYPEDEF:
	    if (ada_prefer_type (SYMBOL_TYPE (syms[i].sym), preferred_type))
	      {
		preferred_index = i;
		preferred_type = SYMBOL_TYPE (syms[i].sym);
	      }
	    break;
	  case LOC_REGISTER:
	  case LOC_ARG:
	  case LOC_REF_ARG:
	  case LOC_REGPARM:
	  case LOC_REGPARM_ADDR:
	  case LOC_LOCAL:
	  case LOC_LOCAL_ARG:
	  case LOC_BASEREG:
	  case LOC_BASEREG_ARG:
          case LOC_COMPUTED:
          case LOC_COMPUTED_ARG:
	    goto NotType;
	  default:
	    break;
	  }
      if (preferred_type != NULL)
	{
	  if (TYPE_CODE (preferred_type) == TYPE_CODE_VOID)
	    error ("`%s' matches only void type name(s)",
		   ada_decode (name));
	  else if (ada_is_object_renaming (syms[preferred_index].sym))
	    {
	      yylval.ssym.sym = syms[preferred_index].sym;
	      *token_type = OBJECT_RENAMING;
	      return segments;
	    }
	  else if (ada_renaming_type (SYMBOL_TYPE (syms[preferred_index].sym))
                   != NULL)
	    {
	      int result;
	      char *renaming
		= ada_simple_renamed_entity (syms[preferred_index].sym);
	      char *new_name
                = (char *) obstack_alloc (&temp_parse_space,
                                          strlen (renaming) + len0
				          - yylval.ssym.stoken.length + 1);
	      strcpy (new_name, renaming);
              xfree (renaming);
	      strcat (new_name, name0 + yylval.ssym.stoken.length);
	      result = name_lookup (new_name, err_name, token_type, depth - 1);
	      if (result > segments)
		error ("Confused by renamed symbol.");
	      return result;
	    }
	  else if (segments == 0)
	    {
	      yylval.tval = preferred_type;
	      *token_type = TYPENAME;
	      return 0;
	    }
	}

      if (segments == 0)
	{
	  type = language_lookup_primitive_type_by_name (current_language,
                                                         current_gdbarch,
                                                         name);
	  if (type == NULL && strcmp ("system__address", name) == 0)
	    type = type_system_address ();
	  if (type != NULL)
	    {
	      /* First check to see if we have a regular definition of this
		 type that just didn't happen to have been read yet.  */
	      int ntypes;
	      struct symbol *sym;
	      char *expanded_name = 
		(char *) alloca (strlen (name) + sizeof ("standard__"));
	      strcpy (expanded_name, "standard__");
	      strcat (expanded_name, name);
	      sym = ada_lookup_symbol (expanded_name, NULL,
				       VAR_DOMAIN, NULL, NULL);
	      if (sym != NULL && SYMBOL_CLASS (sym) == LOC_TYPEDEF)
		type = SYMBOL_TYPE (sym);
					  
	      yylval.tval = type;
	      *token_type = TYPENAME;
	      return 0;
	    }
	}

    NotType:
      if (nsyms == 1)
	{
	  *token_type = NAME;
	  yylval.ssym.sym = syms[0].sym;
	  yylval.ssym.msym = NULL;
	  yylval.ssym.block = syms[0].block;
	  return segments;
	}
      else if (nsyms == 0) {
	int i;
	yylval.ssym.msym = ada_lookup_simple_minsym (name);
	if (yylval.ssym.msym != NULL)
	  {
	    yylval.ssym.sym = NULL;
	    yylval.ssym.block = NULL;
            *token_type = NAME;
	    return segments;
	  }

	if (segments == 0 
	    && strncmp (name, "standard__", sizeof ("standard__") - 1) == 0)
	  error ("No definition of \"%s\" found.", err_name);

	for (i = yylval.ssym.stoken.length - 1; i > 0; i -= 1)
	  {
            if (name[i] == '.')
	      {
		name[i] = '\0';
		yylval.ssym.stoken.length = i;
		break;
	      }
	    else if (name[i] == '_' && name[i-1] == '_')
	      {
		i -= 1;
		name[i] = '\0';
		yylval.ssym.stoken.length = i;
		break;
	      }
	  }
	if (i <= 0)
	  {
	    if (!have_full_symbols () && !have_partial_symbols ()
		&& left_block_context == NULL)
	      error ("No symbol table is loaded.  Use the \"file\" command.");
	    if (left_block_context == NULL)
	      error ("No definition of \"%s\" in current context.",
		     err_name);
	    else
	      error ("No definition of \"%s\" in specified context.",
		     err_name);
	  }
      }
      else
	{
	  *token_type = NAME;
	  yylval.ssym.sym = NULL;
	  yylval.ssym.msym = NULL;
	  if (left_block_context == NULL)
	    yylval.ssym.block = expression_context_block;
	  else
	    yylval.ssym.block = left_block_context;
	  return segments;
	}
    }
}

/* Returns the position within STR of the '.' in a
   '.{WHITE}*all' component of a dotted name, or -1 if there is none.  */
static int
find_dot_all (const char *str)
{
  int i;
  for (i = 0; str[i] != '\000'; i += 1)
    {
      if (str[i] == '.')
	{
	  int i0 = i;
	  do
	    i += 1;
	  while (isspace (str[i]));
	  if (strcmp (str+i, "all") == 0
	      && ! isalnum (str[i+3]) && str[i+3] != '_')
	    return i0;
	}
    }
  return -1;
}

/* Returns non-zero iff string SUBSEQ matches a subsequence of STR, ignoring
   case.  */

static int
subseqMatch (const char *subseq, const char *str)
{
  if (subseq[0] == '\0')
    return 1;
  else if (str[0] == '\0')
    return 0;
  else if (tolower (subseq[0]) == tolower (str[0]))
    return subseqMatch (subseq+1, str+1) || subseqMatch (subseq, str+1);
  else
    return subseqMatch (subseq, str+1);
}


static struct { const char *name; int code; }
attributes[] = {
  { "address", TICK_ADDRESS },
  { "unchecked_access", TICK_ACCESS },
  { "unrestricted_access", TICK_ACCESS },
  { "access", TICK_ACCESS },
  { "first", TICK_FIRST },
  { "last", TICK_LAST },
  { "length", TICK_LENGTH },
  { "max", TICK_MAX },
  { "min", TICK_MIN },
  { "modulus", TICK_MODULUS },
  { "pos", TICK_POS },
  { "range", TICK_RANGE },
  { "size", TICK_SIZE },
  { "tag", TICK_TAG },
  { "val", TICK_VAL },
  { NULL, -1 }
};

/* Return the syntactic code corresponding to the attribute name or
   abbreviation STR.  */

static int
processAttribute (const char *str)
{
  int i, k;

  for (i = 0; attributes[i].code != -1; i += 1)
    if (strcasecmp (str, attributes[i].name) == 0)
      return attributes[i].code;

  for (i = 0, k = -1; attributes[i].code != -1; i += 1)
    if (subseqMatch (str, attributes[i].name))
      {
	if (k == -1)
	  k = i;
	else
	  error ("ambiguous attribute name: `%s'", str);
      }
  if (k == -1)
    error ("unrecognized attribute: `%s'", str);

  return attributes[k].code;
}

int
yywrap(void)
{
  return 1;
}

/* Dummy definition to suppress warnings about unused static definitions. */
typedef void (*dummy_function) ();
dummy_function ada_flex_use[] = 
{ 
  (dummy_function) yyunput
};
